package kr.co.vividnext.sodalive.admin.calculate

import com.querydsl.core.annotations.QueryProjection
import kr.co.vividnext.sodalive.can.use.CanUsage
import kotlin.math.roundToInt

data class GetCalculateLiveQueryData @QueryProjection constructor(
    val email: String,
    val nickname: String,
    val date: String,
    val title: String,
    // 유료방 입장 금액
    val entranceFee: Int,
    // 코인 사용 구분
    val canUsage: CanUsage,
    // 참여인원
    val memberCount: Long,
    // 합계
    val totalAmount: Int
) {
    fun toGetCalculateLiveResponse(): GetCalculateLiveResponse {
        val canUsageStr = when (canUsage) {
            CanUsage.LIVE -> {
                "유료"
            }

            CanUsage.SPIN_ROULETTE -> {
                "룰렛"
            }

            else -> {
                "후원"
            }
        }

        val numberOfPeople = if (canUsage == CanUsage.LIVE) {
            memberCount.toInt()
        } else {
            0
        }

        // 원화 = totalCoin * 100 ( 캔 1개 = 100원 )
        val totalKrw = totalAmount * 100

        // 결제수수료 : 6.6%
        val paymentFee = totalKrw * 0.066f

        // 정산금액 = (원화 - 결제수수료) 의 70%
        val settlementAmount = (totalKrw.toFloat() - paymentFee) * 0.7

        // 원천세 = 정산금액의 3.3%
        val tax = settlementAmount * 0.033

        // 입금액
        val depositAmount = settlementAmount - tax

        return GetCalculateLiveResponse(
            email = email,
            nickname = nickname,
            date = date,
            title = title,
            entranceFee = entranceFee,
            canUsageStr = canUsageStr,
            numberOfPeople = numberOfPeople,
            totalAmount = totalAmount,
            totalKrw = totalKrw,
            paymentFee = paymentFee.roundToInt(),
            settlementAmount = settlementAmount.roundToInt(),
            tax = tax.roundToInt(),
            depositAmount = depositAmount.roundToInt()
        )
    }
}
